from rest_framework import serializers
from .models import Organization
from typing import List


class OrganizationSerializer(serializers.ModelSerializer):
    class Meta:
        model = Organization
        fields = ("networks", "name", "email", "phone", "keys")


class SuccessOrganizationsResponseSerializer(serializers.Serializer):
    response = serializers.CharField(max_length=2)


class DIDSerializer(serializers.Serializer):
    did = serializers.CharField(max_length=255)
    network_name = serializers.CharField(max_length=255)


class CreateDIDResponseSerializer(serializers.Serializer):
    dids = List[DIDSerializer]


class CreateDIDSerializer(serializers.Serializer):
    create_alastria_tx = serializers.CharField(max_length=1255)
    subject_address = serializers.CharField(max_length=255)
    email = serializers.CharField(max_length=255)


class OperatorDIDSerializer(serializers.Serializer):
    did = serializers.CharField(max_length=255)
    email = serializers.CharField(max_length=255)
    network_name = serializers.CharField(max_length=255)



# Generated by Django 4.1.3 on 2023-05-02 15:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('organizations', '0008_alter_organization_networks'),
    ]

    operations = [
        migrations.AddField(
            model_name='issuer',
            name='name',
            field=models.CharField(blank=True, max_length=2500),
        ),
    ]



from __future__ import annotations
from abc import ABC, abstractmethod
from .models import Organization
from typing import List
from network_service_client.client import NetworksNames


class AOrganizationsService(ABC):
    @staticmethod
    @abstractmethod
    def get_organization_by_operator_email(email: str) -> Organization | None:
        ...

    @staticmethod
    @abstractmethod
    def set_did(email: str, did: str, network_name: str) -> None:
        ...

    @staticmethod
    @abstractmethod
    def get_did(
        create_alastria_tx: str,
        subject_address: str,
        email: str,
    ) -> List[dict]:
        ...



from django.dispatch import receiver
from .models import (
    Organization,
    OrganizationKeys,
    UserProxy,
    OrganizationOnboardingNotification,
    Issuer,
)
from django.db.models.signals import post_save
from mnemonic import Mnemonic
from eth_account import Account
from organizations.tasks import create_organization_did, register_issuer_task
from django.conf import settings
from django.contrib.auth.forms import PasswordResetForm
from django.http.request import HttpRequest
from django.core import mail
from templates_service_client.client import Client
from templates_service_client.validators import OperatorOnboardingValidator
from .service import OrganizationsService
from django.utils.timezone import now
from django.utils.html import strip_tags


@receiver(post_save, sender=UserProxy)
def post_save_profile(sender, instance: UserProxy, **kwargs):
    if kwargs.get("created") and settings.OPERATOR_AUTOMATIC_STAFF:
        instance.is_staff = True
        instance.save()


@receiver(post_save, sender=OrganizationOnboardingNotification)
def post_save_onboarding_request(
    sender, instance: OrganizationOnboardingNotification, **kwargs
):
    template_client = Client(settings.TEMPLATES_SERVICE_HOST)
    html_message = template_client.get_operators_onboarding_template_rendered(
        OperatorOnboardingValidator(
            project=settings.TEMPLATES_PROJECT_NAME,
            organization_name=instance.organization.name,
            app_path=settings.APP_PATH,
            web_path=settings.WEB_PATH,
        )
    ).text.replace("\n", "")
    try:
        mail.send_mail(
            "Onboarding",
            strip_tags(html_message),
            settings.DEFAULT_FROM_EMAIL,
            [instance.operator.email],
            html_message=html_message,
        )
    except Exception as e:
        print(str(e))  # TODO MAYBE ADD THIS THO QUEUE ?


@receiver(post_save, sender=Issuer)
def post_save_issuer(sender, instance: Issuer, **kwargs):
    if kwargs.get("created"):
        register_issuer_task.delay(instance.organization.id)


@receiver(post_save, sender=Organization)
def post_save_organization(sender, instance: Organization, **kwargs):
    if kwargs.get("created"):
        mnemo = Mnemonic("english")
        words = mnemo.generate(strength=256)
        mnemo.to_seed(words)
        mnemo.to_entropy(words)
        new_organization_acct = Account.create(words)
        organization_keys = OrganizationKeys(
            address=new_organization_acct.address,
            private_key=new_organization_acct.key.hex(),
            public_key=new_organization_acct._key_obj.public_key,
            mnemonic=words,
        )
        organization_keys.save()
        instance.keys = organization_keys
        instance.save()
        create_organization_did.delay(instance.pk)
    else:
        for operator in instance.operators.all():  # TODO PUT IT IN A TASK
            if not OrganizationOnboardingNotification.objects.filter(
                organization=instance, operator=operator
            ).first():
                OrganizationOnboardingNotification(
                    organization=instance, operator=operator, sended_at=now()
                ).save()



from __future__ import annotations
from .abstractions import AOrganizationsService
from .models import Organization, OperatorDID, UserProxy, Subject, SubjectDID
from django.conf import settings
from network_service_client.client import (
    Client as NetworkClient,
    Network as NetworkDTO,
    NetworksNames,
)
from organizations.did_factory.models import FactoryArgsModel
from organizations.did_factory.factory import Creator
from typing import List


class OrganizationsService(AOrganizationsService):
    @staticmethod
    def get_organization_by_operator_email(email: str) -> Organization | None:
        return Organization.objects.filter(operators__email=email).first()

    @staticmethod
    def set_did(email: str, did: str, network_name: str) -> None:
        subject: Subject | None = Subject.objects.filter(email=email).first()
        operator: UserProxy | None = UserProxy.objects.filter(email=email).first()

        if subject:
            subject_did = SubjectDID(
                network_name=network_name, subject=subject, did=did
            )
            subject_did.save()
        if operator:
            operator_did = OperatorDID(
                network_name=network_name, operator=operator, did=did
            )
            operator_did.save()

    @staticmethod
    def get_did(
        create_alastria_tx: str,
        subject_address: str,
        email: str,
    ) -> List[dict]:
        response: List[dict] = []

        organization: Organization = Organization.objects.filter(
            operators__email=email
        ).first()  # TODO ADD SUPPORT FOR SUBJECTS AND INTERMEDIARY

        for net in organization.networks:
            network_data: NetworkDTO = NetworkClient(
                service_host=settings.NETWORK_SERVICE_HOST
            ).get_network_by_name(NetworksNames[net])
            props = FactoryArgsModel(net=network_data)
            context = Creator().create_object(props).request()

            did: str = context.create_did_from_external_source(
                create_alastria_tx,
                subject_address,
                settings.ISSUER_ADDRESS,
                settings.ISSUER_PRIVATE_KEY,
            )
            OrganizationsService.set_did(email, did, net)
            response.append({"network_name": net, "did": did})

        return response



from django.apps import AppConfig


class OrganizationsConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "organizations"

    def ready(self) -> None:
        import organizations.signals



from django.urls import path
from .views import OrganizationsView
from django.conf import settings
from rest_framework.routers import SimpleRouter


router = SimpleRouter()

router.register(r"organizations", OrganizationsView, "Organizations")

urlpatterns = router.urls



from django.forms import ModelForm
from .models import UserProxy
from django import forms
from uuid import uuid4


def email_unique(value):
    if UserProxy.objects.filter(email=value).first():
        raise forms.ValidationError("That email already exists.")


class UserProxyForm(ModelForm):
    username = forms.CharField(max_length=255, required=False, empty_value=str(uuid4()))
    first_name = forms.CharField(max_length=150, required=False)
    last_name = forms.CharField(max_length=150, required=False)
    email = forms.EmailField(required=True, validators=[email_unique])

    class Meta:
        model = UserProxy
        fields = ["username", "first_name", "last_name", "email"]



from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import gettext_lazy as _
from multiselectfield import MultiSelectField
from network_service_client.client import NetworksNames


class UserProxy(User):
    class Meta:
        proxy = True
        verbose_name = _("Operator")
        verbose_name_plural = _("Operators")

    def __str__(self):
        return f"{self.username} - {self.email}"


class OrganizationKeys(models.Model):
    address = models.CharField(max_length=100)
    private_key = models.CharField(max_length=500)
    public_key = models.CharField(max_length=500)
    mnemonic = models.TextField()

    def __str__(self) -> str:
        return self.address


class Organization(models.Model):
    networks = MultiSelectField(
        choices=NetworksNames.choices(), max_choices=2, max_length=1000
    )
    name = models.CharField(max_length=255, unique=True)
    email = models.EmailField(blank=True, null=True)
    phone = models.CharField(max_length=255, blank=True, null=True)
    keys = models.ForeignKey(
        OrganizationKeys, on_delete=models.SET_NULL, blank=True, null=True
    )
    operators = models.ManyToManyField(UserProxy, blank=True)

    def __str__(self) -> str:
        return self.name


class OperatorDID(models.Model):
    network_name = models.CharField(max_length=2500)
    operator = models.ForeignKey(UserProxy, on_delete=models.CASCADE)
    did = models.CharField(max_length=255)

    def __str__(self) -> str:
        return f"{self.operator.email} - {self.network_name} - {self.did}"


class OrganizationDID(models.Model):
    network_name = models.CharField(max_length=2500)
    organization = models.ForeignKey(Organization, on_delete=models.CASCADE)
    did = models.CharField(max_length=255)

    def __str__(self) -> str:
        return f"{self.network_name} - {self.organization.name}"


class Issuer(models.Model):
    name = models.CharField(max_length=2500, null=False, blank=True)
    organization = models.ForeignKey(Organization, on_delete=models.CASCADE)
    active = models.BooleanField(default=False)

    def __str__(self) -> str:
        return f"{self.name} - {self.organization.name}"


class Intermediary(models.Model):
    organization = models.ForeignKey(Organization, on_delete=models.CASCADE)
    active = models.BooleanField(default=True)

    def __str__(self) -> str:
        return self.organization.name


class Subject(models.Model):
    email = models.EmailField(_("Email"), max_length=200, null=True, blank=True)
    givenName = models.CharField(max_length=255, blank=True)
    familyName = models.CharField(max_length=255, blank=True)
    birthDate = models.DateField(blank=True, null=True)
    intermediary = models.ForeignKey(Intermediary, on_delete=models.CASCADE)

    class Meta:
        verbose_name = _("Subject")
        verbose_name_plural = _("Subjects")

    def __str__(self) -> str:
        display_text = self.email
        return display_text


class SubjectDID(models.Model):
    network_name = models.CharField(max_length=2500)
    subject = models.ForeignKey(Subject, on_delete=models.CASCADE)
    did = models.CharField(max_length=255)

    def __str__(self) -> str:
        return self.organization.name


class OrganizationOnboardingNotification(models.Model):
    organization = models.ForeignKey(Organization, on_delete=models.CASCADE)
    operator = models.ForeignKey(UserProxy, on_delete=models.CASCADE)
    sended_at = models.DateTimeField()



from django.contrib import admin
from .models import (
    Intermediary,
    Organization,
    OrganizationDID,
    OrganizationKeys,
    Issuer,
    UserProxy,
    OperatorDID,
    Subject,
    SubjectDID,
    OrganizationOnboardingNotification,
)
from .enums import OrganizationsSwitches
import waffle
from .forms import UserProxyForm


class OperatorDIDAdmin(admin.ModelAdmin):
    model = OperatorDID

    def get_model_perms(self, request):

        if not waffle.switch_is_active(OrganizationsSwitches.Organizations.value):
            return {}

        return super(OperatorDIDAdmin, self).get_model_perms(request)


class SubjectDIDAdmin(admin.ModelAdmin):
    model = SubjectDID

    def get_model_perms(self, request):

        if (
            not waffle.switch_is_active(OrganizationsSwitches.Organizations.value)
            or waffle.switch_is_active(OrganizationsSwitches.Organizations.value)
            and not waffle.switch_is_active(
                OrganizationsSwitches.OrganizationsIntermediary.value
            )
        ):
            return {}

        return super(SubjectDIDAdmin, self).get_model_perms(request)


class SubjectAdmin(admin.ModelAdmin):
    model = Subject

    def get_model_perms(self, request):

        if (
            not waffle.switch_is_active(OrganizationsSwitches.Organizations.value)
            or waffle.switch_is_active(OrganizationsSwitches.Organizations.value)
            and not waffle.switch_is_active(
                OrganizationsSwitches.OrganizationsIntermediary.value
            )
        ):
            return {}

        return super(SubjectAdmin, self).get_model_perms(request)


class IntermediaryAdmin(admin.ModelAdmin):
    model = Intermediary

    def get_model_perms(self, request):

        if (
            not waffle.switch_is_active(OrganizationsSwitches.Organizations.value)
            or waffle.switch_is_active(OrganizationsSwitches.Organizations.value)
            and not waffle.switch_is_active(
                OrganizationsSwitches.OrganizationsIntermediary.value
            )
        ):
            return {}

        return super(IntermediaryAdmin, self).get_model_perms(request)


class OrganizationAdmin(admin.ModelAdmin):
    model = Organization
    list_filter = ['networks']
    search_fields = ['name', 'email', 'phone',]
    
    def get_model_perms(self, request):

        if not waffle.switch_is_active(OrganizationsSwitches.Organizations.value):
            return {}

        return super(OrganizationAdmin, self).get_model_perms(request)


class OrganizationDIDAdmin(admin.ModelAdmin):
    model = OrganizationDID
    list_filter = ['network_name', 'organization']
    search_fields = ['network_name', 'organization', 'did',]
    
    def get_model_perms(self, request):

        if not waffle.switch_is_active(OrganizationsSwitches.Organizations.value):
            return {}

        return super(OrganizationDIDAdmin, self).get_model_perms(request)


class OrganizationKeysAdmin(admin.ModelAdmin):
    model = OrganizationKeys

    def get_model_perms(self, request):

        if not waffle.switch_is_active(OrganizationsSwitches.Organizations.value):
            return {}

        return super(OrganizationKeysAdmin, self).get_model_perms(request)


class IssuerAdmin(admin.ModelAdmin):
    model = Issuer
    readonly_fields = ["active"]
    list_filter = ['organization']


    def get_model_perms(self, request):

        if (
            not waffle.switch_is_active(OrganizationsSwitches.Organizations.value)
            or waffle.switch_is_active(OrganizationsSwitches.Organizations.value)
            and not waffle.switch_is_active(
                OrganizationsSwitches.OrganizationsIssuer.value
            )
        ):
            return {}

        return super(IssuerAdmin, self).get_model_perms(request)


class UserProxyAdmin(admin.ModelAdmin):
    form = UserProxyForm

    def get_model_perms(self, request):

        if not waffle.switch_is_active(OrganizationsSwitches.Organizations.value):
            return {}

        return super(UserProxyAdmin, self).get_model_perms(request)


class OrganizationOnboardingNotificationAdmin(admin.ModelAdmin):
    model = OrganizationOnboardingNotification
    list_filter = ['organization']
    
    def get_model_perms(self, request):

        if not waffle.switch_is_active(OrganizationsSwitches.Organizations.value):
            return {}

        return super(OrganizationOnboardingNotificationAdmin, self).get_model_perms(
            request
        )


admin.site.register(
    OrganizationOnboardingNotification, OrganizationOnboardingNotificationAdmin
)
admin.site.register(SubjectDID, SubjectDIDAdmin)
admin.site.register(Subject, SubjectAdmin)
admin.site.register(OperatorDID, OperatorDIDAdmin)
admin.site.register(Intermediary, IntermediaryAdmin)
admin.site.register(Organization, OrganizationAdmin)
admin.site.register(OrganizationDID, OrganizationDIDAdmin)
admin.site.register(OrganizationKeys, OrganizationKeysAdmin)
admin.site.register(Issuer, IssuerAdmin)
admin.site.register(UserProxy, UserProxyAdmin)



# Generated by Django 4.1.3 on 2023-01-30 10:04

from django.db import migrations, models
import django.db.models.deletion
import multiselectfield.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ("organizations", "0005_subject_alter_organization_networks_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="organization",
            name="networks",
            field=multiselectfield.db.fields.MultiSelectField(
                choices=[
                    ("AlastriaDefaultName", "Alastria"),
                    ("LacchainDefaultName", "Lacchain"),
                ],
                max_length=1000,
            ),
        ),
        migrations.CreateModel(
            name="OrganizationOnboardingNotification",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("sended_at", models.DateTimeField()),
                (
                    "operator",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="organizations.userproxy",
                    ),
                ),
                (
                    "organization",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="organizations.organization",
                    ),
                ),
            ],
        ),
    ]



# Generated by Django 4.1.3 on 2023-01-27 07:31

from django.db import migrations, models
import django.db.models.deletion
import multiselectfield.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ("organizations", "0004_alter_organization_email_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="Subject",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "email",
                    models.EmailField(
                        blank=True, max_length=200, null=True, verbose_name="Email"
                    ),
                ),
                ("givenName", models.CharField(blank=True, max_length=255)),
                ("familyName", models.CharField(blank=True, max_length=255)),
                ("birthDate", models.DateField(blank=True, null=True)),
                (
                    "intermediary",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="organizations.intermediary",
                    ),
                ),
            ],
            options={
                "verbose_name": "Subject",
                "verbose_name_plural": "Subjects",
            },
        ),
        migrations.AlterField(
            model_name="organization",
            name="networks",
            field=multiselectfield.db.fields.MultiSelectField(
                choices=[("AlastriaDefaultName", "Alastria")], max_length=1000
            ),
        ),
        migrations.AlterField(
            model_name="organization",
            name="operators",
            field=models.ManyToManyField(blank=True, to="organizations.userproxy"),
        ),
        migrations.CreateModel(
            name="SubjectDID",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("network_name", models.CharField(max_length=2500)),
                ("did", models.CharField(max_length=255)),
                (
                    "subject",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="organizations.subject",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="OperatorDID",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("network_name", models.CharField(max_length=2500)),
                ("did", models.CharField(max_length=255)),
                (
                    "operator",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="organizations.userproxy",
                    ),
                ),
            ],
        ),
    ]



from __future__ import annotations
from celery import shared_task
from django.conf import settings
from network_service_client.client import (
    Client as NetworkClient,
    Network as NetworkDTO,
    NetworksNames,
)
from organizations.did_factory.models import FactoryArgsModel
from organizations.did_factory.factory import Creator
from organizations.models import Organization, OrganizationDID, Issuer


@shared_task
def create_organization_did(organization_id: int) -> None:
    organization = Organization.objects.get(pk=organization_id)
    for net in organization.networks:
        network_data: NetworkDTO = NetworkClient(
            service_host=settings.NETWORK_SERVICE_HOST
        ).get_network_by_name(NetworksNames[net])
        props = FactoryArgsModel(net=network_data)
        context = Creator().create_object(props).request()
        did: str = context.create_did(
            organization.keys.address,
            settings.ISSUER_ADDRESS,
            str(organization.keys.public_key),
            settings.ISSUER_PRIVATE_KEY,
            organization.keys.private_key,  # TODO: encrypt this
        )

        organization_did = OrganizationDID(
            network_name=NetworksNames[net],
            organization=organization,
            did=did,
        )

        organization_did.save()


@shared_task
def register_issuer_by_network_task(organization_id: int, net: str) -> None:
    organization = Organization.objects.get(pk=organization_id)
    issuer: Issuer = Issuer.objects.get(organization=organization)
    organization_did: OrganizationDID | None = OrganizationDID.objects.filter(
        organization=organization, network_name=NetworksNames[net]
    ).first()
    if not organization_did:
        raise Exception(
            f"You cannot register an organization without DID as an issuer, organization: {organization.id} network: {NetworksNames[net]}"
        )
    network_data: NetworkDTO = NetworkClient(
        service_host=settings.NETWORK_SERVICE_HOST
    ).get_network_by_name(NetworksNames[net])
    props = FactoryArgsModel(net=network_data)
    context = Creator().create_object(props).request()
    context.register_issuer(
        organization.keys.address,
        settings.ISSUER_ADDRESS,
        settings.ISSUER_PRIVATE_KEY,
    )
    issuer.active = True
    issuer.save()


@shared_task
def register_issuer_task(organization_id: int) -> None:
    organization = Organization.objects.get(pk=organization_id)
    for net in organization.networks:
        register_issuer_by_network_task.delay(organization_id, net)



from .abstractions import AFactory, AContext
from .strategy import (
    AlastriaNetworkStrategy,
    EbsiNetworkStrategy,
    LacchainNetworkStrategy,
)
from .context import Context
from network_service_client.client import NetworksNames
from .models import FactoryArgsModel


class Creator(AFactory):
    @staticmethod
    def create_object(props: FactoryArgsModel) -> AContext:

        if props.net.name == NetworksNames.AlastriaDefaultName:
            return Context(AlastriaNetworkStrategy(props))
        if props.net.name == NetworksNames.LacchainDefaultName:
            return Context(LacchainNetworkStrategy(props))
        if props.net.name == NetworksNames.EbsiDefaultName:
            return Context(EbsiNetworkStrategy(props))
        raise Exception("Cant find any strategy.")



# Generated by Django 4.1.3 on 2023-01-24 10:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("organizations", "0003_alter_organization_networks"),
    ]

    operations = [
        migrations.AlterField(
            model_name="organization",
            name="email",
            field=models.EmailField(blank=True, max_length=254, null=True),
        ),
        migrations.AlterField(
            model_name="organization",
            name="operators",
            field=models.ManyToManyField(
                null=True, to="organizations.userproxy"
            ),
        ),
        migrations.AlterField(
            model_name="organization",
            name="phone",
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]



from enum import Enum


class OrganizationsSwitches(str, Enum):
    Organizations = "organizations manager"
    OrganizationsIssuer = "organizations issuer manager"
    OrganizationsIntermediary = "organizations intermediary manager"



# Generated by Django 4.1.3 on 2023-01-18 15:06

import django.contrib.auth.models
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
    ]

    operations = [
        migrations.CreateModel(
            name="Organization",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("email", models.EmailField(max_length=254)),
                ("phone", models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name="OrganizationKeys",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("address", models.CharField(max_length=100)),
                ("private_key", models.CharField(max_length=500)),
                ("public_key", models.CharField(max_length=500)),
                ("mnemonic", models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name="UserProxy",
            fields=[],
            options={
                "verbose_name": "Operator",
                "verbose_name_plural": "Operators",
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("auth.user",),
            managers=[
                ("objects", django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name="OrganizationDID",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("network_name", models.CharField(max_length=2500)),
                ("did", models.CharField(max_length=255)),
                (
                    "organization",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="organizations.organization",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="organization",
            name="keys",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="organizations.organizationkeys",
            ),
        ),
        migrations.AddField(
            model_name="organization",
            name="operators",
            field=models.ManyToManyField(to="organizations.userproxy"),
        ),
        migrations.CreateModel(
            name="Issuer",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("active", models.BooleanField(default=True)),
                (
                    "organization",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="organizations.organization",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Intermediary",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("active", models.BooleanField(default=True)),
                (
                    "organization",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="organizations.organization",
                    ),
                ),
            ],
        ),
    ]



from rest_framework.viewsets import ViewSet
from rest_framework.permissions import AllowAny
from drf_yasg import openapi
from drf_yasg.utils import swagger_auto_schema
from rest_framework.decorators import action
from rest_framework.response import Response
from waffle.decorators import waffle_switch
from django.http.response import HttpResponseBadRequest
from .enums import OrganizationsSwitches
from .serializers import (
    DIDSerializer,
    OperatorDIDSerializer,
    OrganizationSerializer,
    SuccessOrganizationsResponseSerializer,
    CreateDIDSerializer,
    CreateDIDResponseSerializer,
)

from .service import OrganizationsService


class OrganizationsView(ViewSet):
    permission_classes = (AllowAny,)

    @swagger_auto_schema(
        method="get",
        manual_parameters=[
            openapi.Parameter(
                "operator_email",
                openapi.IN_QUERY,
                description="",
                type=openapi.TYPE_STRING,
            )
        ],
        responses={200: openapi.Response("", OrganizationSerializer)},
    )
    @waffle_switch(OrganizationsSwitches.Organizations.value)
    @action(detail=False, methods=["get"])
    def get_organization_by_operator_email(self, request):
        operator_email: str = request.GET["operator_email"]
        organization = OrganizationsService.get_organization_by_operator_email(
            operator_email
        )
        if organization:
            return Response(OrganizationSerializer(organization).data)

        return HttpResponseBadRequest("Invalid email.")

    @swagger_auto_schema(
        method="post",
        request_body=CreateDIDSerializer,
        responses={200: openapi.Response("", CreateDIDResponseSerializer)},
    )
    @waffle_switch(OrganizationsSwitches.Organizations.value)
    @action(detail=False, methods=["post"])
    def get_did(self, request):
        create_alastria_tx: str = request.data["create_alastria_tx"]
        subject_address: str = request.data["subject_address"]
        email: str = request.data["email"]
        return Response(
            OrganizationsService.get_did(create_alastria_tx, subject_address, email)
        )

    @swagger_auto_schema(
        method="post",
        request_body=OperatorDIDSerializer,
        responses={200: openapi.Response("", SuccessOrganizationsResponseSerializer)},
    )
    @waffle_switch(OrganizationsSwitches.Organizations.value)
    @action(detail=False, methods=["post"])
    def set_did(self, request):
        email: str = request.data["email"]
        did: str = request.data["did"]
        network_name: str = request.data["network_name"]
        OrganizationsService.set_did(email, did, network_name)
        return Response({"response": "OK"})



# Generated by Django 4.1.3 on 2023-03-15 15:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('organizations', '0006_alter_organization_networks_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='issuer',
            name='active',
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 4.1.3 on 2023-04-27 13:44

from django.db import migrations
import multiselectfield.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ('organizations', '0007_alter_issuer_active'),
    ]

    operations = [
        migrations.AlterField(
            model_name='organization',
            name='networks',
            field=multiselectfield.db.fields.MultiSelectField(choices=[('AlastriaDefaultName', 'Alastria'), ('LacchainDefaultName', 'Lacchain'), ('EbsiDefaultName', 'Ebsi')], max_length=1000),
        ),
    ]
